/*************************************************************
*
* Description:
*	ATMega128 - Example program using ICC-AVR
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#define	TASK_STK_SIZE   512
#define MEM_POOL_SIZE   1024

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];

U08 pool[MEM_POOL_SIZE];

U08 OS_SystemTicksCnt;
U08 ncounts;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
OS_SEM LedSem;
OS_TMR LedTimer;
OS_TMR ROnceTimer1;
OS_TMR ROnceTimer2;
#endif

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

void main(void)
{
    OS_ENTER_CRITICAL();
    XDIV  = 0x00;  //xtal divider
    XMCRA = 0x00;  //external memory
    PORTB = 0xFF;
    DDRB  = 0x01;
    MCUCR = 0xC0;
    EICRA = 0x00;  //extended ext ints
    EICRB = 0x00;  //extended ext ints
    EIMSK = 0x00;
    TIMSK = 0x01;  //timer interrupt sources
    ETIMSK = 0x00; //extended timer interrupt sources
    OS_EXIT_CRITICAL();

    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], TASK_STK_SIZE, 11);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], TASK_STK_SIZE, 13);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], TASK_STK_SIZE, 12);
    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif

    OS_Start();
}

void OSTickISR2(void)
{
    TCNT0   = ncounts;                                     // set count (reload)
#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
}

void AppTickInit(void)
{
    OS_SystemTicksCnt = 0;
    OS_ENTER_CRITICAL();              // we use Timer0 normal, prescaler 32, 100Hz for OS_Timer (102.4Hz)
    TCCR0   = 0x00;                   // stop
    ASSR    = 0x08;                   // set async mode
    ncounts = 0 - (32768 / 32 / OS_TIMER_TICKS_PER_SEC); // calculate counter for 32.768kHz clock
    TCNT0   = ncounts;                //set count (needed in ISR too, for reloading every interrupt)
    OCR0    = 0x0A;
    TCCR0   = 0x03;                   //start timer
    OS_EXIT_CRITICAL();
}

void Task1(void *Tdata)
{
    U08 returnOk;
    U32 event, i;
    U08 OS_HUGE *ptr;

    if(Tdata != 0) while(1);
    i=0;
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);
    returnOk=OS_MemFree((OS_MEM OS_HUGE *)ptr);
    while(1) {
        while(i<250) i++;                               // no Task-switch to MUTEX
        i=0;
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // first ok. - secound suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    i=0;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // suspending
        if(!returnOk) {
            while(i<250) i++;                           // no Taskswitch to Task1 or Task3
            i=0;
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                  // stope LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);              // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);           // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}

void Task3(void *Tdata)
{
    U16 i;
    U08 LEDs = 0xFE;                                       // first ON bit-0

    Tdata = Tdata;                                         // install timer & callback-function
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
    i=0;
    while(1) {
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timmer-triggert signal
        PORTB = LEDs;
        OS_TimeDly(3);                      // test timer-wakeup
        LEDs ^= 0x01;                       // Toggle LED
        i++;
    }
}
#else
void Task3(void *Tdata)
{
    U16 i;
    U08 LEDs = 0xFE;                        // first ON bit-0

    AppTickInit();
    Tdata = Tdata;
    i=0;
    while(1) {
        PORTB = LEDs;
        OS_TimeDly(3);                      // test timer-wakeup
        LEDs ^= 0x01;                       // Toggle LED
        i++;
    }
}
#endif

