/*************************************************************
*
* Description:
*   FE310 RISC-V - Example program using FreedomStudio
*
* Author[s]:
*   Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
 OS_SEM LedSem;
 OS_TMR LedTimer;
 OS_TMR ROnceTimer1;
 OS_TMR ROnceTimer2;
#endif

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

/***************************************************************/
#define RTC_FREQ 32768
void SysTick_init(void)
{
    clear_csr(mie, MIP_MEIP);                              // extern PLIC         off
    clear_csr(mie, MIP_MTIP);                              // Timer    interrupts off
    clear_csr(mie, MIP_MSIP);                              // Software interrupts off
    CLINT->mtimecmp = CLINT->mtime + (RTC_FREQ / OS_TIMER_TICKS_PER_SEC);
    set_csr(mie, MIP_MSIP);                                // software interrupts on
    set_csr(mie, MIP_MTIP);                                // Timer    interrupts on
//    set_csr(mie, MIP_MEIP);                                // extern PLIC         on
    set_csr(mstatus, MSTATUS_MPP | MSTATUS_MPIE);          // General
}

void SysTick_update(void)
{
    clear_csr(mie, MIP_MTIP);                              // Timer    interrupts off
    CLINT->mtimecmp = CLINT->mtime + (RTC_FREQ / OS_TIMER_TICKS_PER_SEC);
    set_csr(mie, MIP_MTIP);                                // Timer    interrupts on
}

void LED_init(void)
{
    GPIO0->iof_en    &= ~GPIO_pin(19);
    GPIO0->out_xor   &= ~GPIO_pin(19);
    GPIO0->output_en |= GPIO_pin(19);
}

void LED_toggle(void)
{
    if(GPIO0->port & GPIO_pin(19))
        GPIO0->port &= ~GPIO_pin(19);
    else
        GPIO0->port |= GPIO_pin(19);
}

#define F_CPU 320000000                 // mpu PLL-clock to use 320MHz
extern void OStrapEntry(void);
void SystemInit(void)
{
    clear_csr(mie, MIP_MEIP);
    write_csr(mtvec, &OStrapEntry);

#ifdef F_CPU	// PLL
    RTC->lfrosccfg  &= ~RTC_lfrosccfg_en;
    PRCI->hfrosccfg  = PRCI_hfrosccfg_hfroscdiv_s(4) | PRCI_hfrosccfg_hfrosctrim_s(16) | PRCI_hfrosccfg_hfroscen;
    while((PRCI->hfrosccfg & PRCI_hfrosccfg_hfroscrdy) == 0);
    PRCI->pllcfg    &= ~PRCI_pllcfg_pllsel;
    SPI0->sckdiv     = 8;               // Slow down the QSPI
    PRCI->plloutdiv  = PRCI_plloutdiv_plloutdivby1 | PRCI_plloutdiv_plloutdiv_s(0);
    PRCI->pllcfg     = PRCI_pllcfg_pllrefsel | PRCI_pllcfg_pllbypass | PRCI_pllcfg_pllr_s(1) | PRCI_pllcfg_pllf_s(PLL_F_VAL) | PRCI_pllcfg_pllq_s(1);
    PRCI->pllcfg    &= ~PRCI_pllcfg_pllbypass;
    uint64_t now = CLINT->mtime;
    while (CLINT->mtime - now < 4);     // wait 100us (4 ticks of RTC)
    while((PRCI->pllcfg & PRCI_pllcfg_plllock) == 0);
    PRCI->pllcfg    |= PRCI_pllcfg_pllsel;
    PRCI->hfrosccfg &= ~PRCI_hfrosccfg_hfroscen;
#endif
}

/***************************************************************/

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    // ---------------------

    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);
#if OS_STK_CHECK_EN
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], &Stk1, 11);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], &Stk2, 13);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], &Stk3, 12);
#else
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], 11);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], 13);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], 12);
#endif
    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // typical RISC-V pC/OS ISR
{
    // ---- HW specific ----
    SysTick_update();
    // ---------------------

#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
}

void AppTickInit(void)
{                                                          // Configure the system tick
    OS_SystemTicksCnt = 0;

    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((OS_MEM OS_HUGE *)ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - second suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPendAbbort(&Mutex1);
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    if(Tdata != NULL) while(1);
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Task-switch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                               // stop LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);                           // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);                        // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
 
    if(Tdata != NULL) while(1);

    // ---- HW specific ----
    LED_init();
    // ---------------------

#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif

    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timmer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        // ---- HW specific ----
        LED_toggle();
        // ---------------------
    }
}
