/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      pC_OS_tmr.c                         *
*                     TIMER SERVICE                        *
***********************************************************/

#if OS_TMR_EN
/*
************************************************************
*                GLOBAL AND LOCAL VARIABLES                *
************************************************************
*/
    OS_TMR OS_HUGE *TMR_FirstPtr;
    OS_SEM          OSTimerSignal;
    OS_STK_TYPE     OSTimerTaskStk[OS_TMR_STK_SIZE];
    U32             OSTimerCnt;
/*
************************************************************
*                LOCAL FUNCTION PROTOTYPES                 *
************************************************************
*/
void OS_FAR OSTaskTimer(void *Tdata) KC51_REENTRANT;
/*
************************************************************
*                      CREATE A TIMER                      *
************************************************************
*/
U08 OS_TimerCreate(OS_TMR *ptmr, U32 time, void(*tFct)(void *), void *tArg, U08 mode) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    OS_TMR OS_HUGE *last_TMR;

    if (time == 0) 
        return (OS_TMR_NO_TIME);
    OS_ENTER_CRITICAL();
    last_TMR          = TMR_FirstPtr;
    if (last_TMR == NULL) {
        TMR_FirstPtr      = ptmr;
        ptmr->OSTPrev     = NULL;
    } else {
        while ((last_TMR != ptmr) && (last_TMR->OSTNext != NULL))
            last_TMR      = last_TMR->OSTNext;
        if (last_TMR == ptmr) {
            OS_EXIT_CRITICAL();
            return (OS_TMR_EXIST);
        }
        last_TMR->OSTNext = ptmr;
        ptmr->OSTPrev     = last_TMR;
    }
    ptmr->OSTNext     = NULL;
    ptmr->OSTFct      = tFct;
    ptmr->OSTArg      = tArg;
    ptmr->OSTValue    = time;
    ptmr->OSTCntEnd   = OSTimerCnt + time;
    ptmr->OSTMode     = mode & (OS_TMR_ENABLE | OS_TMR_MODE_M);
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                      DELETE A TIMER                      *
************************************************************
*/
U08 OS_TimerDelete(OS_TMR *ptmr) KC51_REENTRANT                         
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    OS_ENTER_CRITICAL();
    if ((ptmr != TMR_FirstPtr) && (ptmr->OSTPrev->OSTNext != ptmr)) {
        OS_EXIT_CRITICAL();
        return (OS_TMR_NOT_EXIST);
    }
    if (ptmr->OSTPrev != NULL)
        ptmr->OSTPrev->OSTNext = ptmr->OSTNext;
    if (ptmr->OSTNext != NULL)
        ptmr->OSTNext->OSTPrev = ptmr->OSTPrev;
    if (ptmr == TMR_FirstPtr)
        TMR_FirstPtr           = ptmr->OSTNext;
    ptmr->OSTMode      = 0;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                   (RE-)START A TIMER                     *
************************************************************
*/
U08 OS_TimerStart(OS_TMR *ptmr, U32 time) KC51_REENTRANT                        
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    OS_ENTER_CRITICAL();
    if ((ptmr != TMR_FirstPtr) && (ptmr->OSTPrev->OSTNext != ptmr)) {
        OS_EXIT_CRITICAL();
        return (OS_TMR_NOT_EXIST);
    }
    ptmr->OSTMode     |= OS_TMR_ENABLE;
    if (time != 0)
        ptmr->OSTValue = time;
    ptmr->OSTCntEnd    = OSTimerCnt + ptmr->OSTValue;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                      STOP A TIMER                        *
************************************************************
*/
U08 OS_TimerStop(OS_TMR *ptmr) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif

    OS_ENTER_CRITICAL();
    if ((ptmr != TMR_FirstPtr) && (ptmr->OSTPrev->OSTNext != ptmr)) {
        OS_EXIT_CRITICAL();
        return (OS_TMR_NOT_EXIST);
    }
    ptmr->OSTMode  &= ~OS_TMR_ENABLE;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                   GET STATE OF A TIMER                   *
************************************************************
*/
U08 OS_TimerGetState(OS_TMR *ptmr) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08 state;
    
    OS_ENTER_CRITICAL();
    if ((ptmr != TMR_FirstPtr) && (ptmr->OSTPrev->OSTNext != ptmr)) {
        OS_EXIT_CRITICAL();
        return (0);
    }
    state  = ptmr->OSTMode;
    OS_EXIT_CRITICAL();
    return (state);
}
/*
************************************************************
*             GET REMAIN TIME OF A RUNNING TIMER           *
************************************************************
*/
U32 OS_TimerGetRemain(OS_TMR *ptmr) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08 time;
    
    OS_ENTER_CRITICAL();
    if ((ptmr != TMR_FirstPtr) && (ptmr->OSTPrev->OSTNext != ptmr)) {
        OS_EXIT_CRITICAL();
        return (0);
    }
    if (!(ptmr->OSTMode & OS_TMR_ENABLE)) {
        OS_EXIT_CRITICAL();
        return (0);
    }
    if (ptmr->OSTCntEnd >= OSTimerCnt)
        time  = ptmr->OSTCntEnd - OSTimerCnt;
    else
        time  = ptmr->OSTCntEnd + (0 - OSTimerCnt);
    OS_EXIT_CRITICAL();
    return (time);
}
/*
************************************************************
*                      TIMER TASK                          *
************************************************************
*/
void OS_FAR OSTaskTimer(void *Tdata) KC51_REENTRANT
{
    OS_TMR OS_HUGE *current_TMR;
    void          (*tFct)(void *);
    void           *tArg;

    Tdata = Tdata;
    while (1) {
        OS_SemPend(&OSTimerSignal, OS_SUSPEND);
        OSTimerCnt++;
        current_TMR = TMR_FirstPtr;
        while (current_TMR != NULL) {
            tFct = NULL;
            tArg = NULL;
            OS_Lock();
            if (current_TMR->OSTMode & OS_TMR_ENABLE) {
                if (current_TMR->OSTCntEnd == OSTimerCnt) {
                    tFct = current_TMR->OSTFct;
                    tArg = current_TMR->OSTArg;
                    if ((current_TMR->OSTMode & OS_TMR_MODE_M) == OS_TMR_CLR) {
                        OS_TimerDelete(current_TMR);
                    } else {
                        if (current_TMR->OSTMode & OS_TMR_CYCL)
                            current_TMR->OSTCntEnd = OSTimerCnt + current_TMR->OSTValue;
                        else
                            current_TMR->OSTMode  &= ~OS_TMR_ENABLE;
                    }
                }
            }
            current_TMR = current_TMR->OSTNext;
            OS_Unlock();
            if (tFct != NULL)
                (*tFct)(tArg);
        }
    }
}
/*
************************************************************
*                  PROCESS TIMER TICK                      *
************************************************************
*/
U08 OSTimerTick(void)
{
    return (OS_SemPost(&OSTimerSignal));
}
#endif /* OS_TMR_EN */

/************************* END ****************************/


